@@ -35,6 +35,9 @@ struct _VinagreConnectionPrivate
   gchar *name;
   gchar *password;
   gchar *desktop_name;
+  gboolean view_only;
+  gboolean scaling;
+  gboolean fullscreen;
 };
 
 enum
@@ -47,9 +50,15 @@ enum
   PROP_PASSWORD,
   PROP_DESKTOP_NAME,
   PROP_BEST_NAME,
-  PROP_ICON
+  PROP_ICON,
+  PROP_VIEW_ONLY,
+  PROP_SCALING,
+  PROP_FULLSCREEN
 };
 
+gint   vinagre_connection_default_port [VINAGRE_CONNECTION_PROTOCOL_INVALID-1] = {5900, 3389};
+gchar* vinagre_connection_protos [VINAGRE_CONNECTION_PROTOCOL_INVALID-1] = {"vnc", "rdp"};
+
 #define VINAGRE_CONNECTION_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VINAGRE_TYPE_CONNECTION, VinagreConnectionPrivate))
 G_DEFINE_TYPE (VinagreConnection, vinagre_connection, G_TYPE_OBJECT);
 
@@ -64,6 +73,9 @@ vinagre_connection_init (VinagreConnection *conn)
   conn->priv->password = NULL;
   conn->priv->name = NULL;
   conn->priv->desktop_name = NULL;
+  conn->priv->view_only = FALSE;
+  conn->priv->scaling = FALSE;
+  conn->priv->fullscreen = FALSE;
 }
 
 static void
@@ -125,6 +137,18 @@ vinagre_connection_set_property (GObject *object, guint prop_id, const GValue *v
 	vinagre_connection_set_desktop_name (conn, g_value_get_string (value));
 	break;
 
+      case PROP_VIEW_ONLY:
+	vinagre_connection_set_view_only (conn, g_value_get_boolean (value));
+	break;
+
+      case PROP_SCALING:
+	vinagre_connection_set_scaling (conn, g_value_get_boolean (value));
+	break;
+
+      case PROP_FULLSCREEN:
+	vinagre_connection_set_fullscreen (conn, g_value_get_boolean (value));
+	break;
+
       default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	break;
@@ -175,6 +199,18 @@ vinagre_connection_get_property (GObject *object, guint prop_id, GValue *value,
 	g_value_set_object (value, vinagre_connection_get_icon (conn));
 	break;
 
+      case PROP_VIEW_ONLY:
+	g_value_set_boolean (value, conn->priv->view_only);
+	break;
+
+      case PROP_SCALING:
+	g_value_set_boolean (value, conn->priv->scaling);
+	break;
+
+      case PROP_FULLSCREEN:
+	g_value_set_boolean (value, conn->priv->fullscreen);
+	break;
+
       default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	break;
@@ -289,6 +325,39 @@ vinagre_connection_class_init (VinagreConnectionClass *klass)
                                                         G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_VIEW_ONLY,
+                                   g_param_spec_boolean ("view-only",
+                                                        "View-only connection",
+	                                                "Whether this connection is a view-only one",
+                                                        FALSE,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_SCALING,
+                                   g_param_spec_boolean ("scaling",
+                                                        "Use scaling",
+	                                                "Whether to use scaling on this connection",
+                                                        FALSE,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
+  g_object_class_install_property (object_class,
+                                   PROP_FULLSCREEN,
+                                   g_param_spec_boolean ("fullscreen",
+                                                        "Full screen connection",
+	                                                "Whether this connection is a view-only one",
+                                                        FALSE,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
 
 }
 
@@ -432,19 +501,26 @@ vinagre_connection_clone (VinagreConnection *conn)
   vinagre_connection_set_password (new_conn, vinagre_connection_get_password (conn));
   vinagre_connection_set_name (new_conn, vinagre_connection_get_name (conn));
   vinagre_connection_set_desktop_name (new_conn, vinagre_connection_get_desktop_name (conn));
+  vinagre_connection_set_view_only (new_conn, vinagre_connection_get_view_only (conn));
+  vinagre_connection_set_scaling (new_conn, vinagre_connection_get_scaling (conn));
+  vinagre_connection_set_fullscreen (new_conn, vinagre_connection_get_fullscreen (conn));
 
   return new_conn;
 }
 
-VinagreConnection *
-vinagre_connection_new_from_string (const gchar *uri, gchar **error_msg)
+gboolean
+vinagre_connection_split_string (const gchar *uri,
+				 gchar **host,
+				 gint *port,
+				 gchar **error_msg)
 {
-  VinagreConnection *conn;
   gchar **server, **url;
-  gint    port;
-  gchar  *host;
+  gint    lport;
+  gchar  *lhost;
 
   *error_msg = NULL;
+  *host = NULL;
+  *port = 0;
 
   url = g_strsplit (uri, "://", 2);
   if (g_strv_length (url) == 2)
@@ -454,28 +530,47 @@ vinagre_connection_new_from_string (const gchar *uri, gchar **error_msg)
 	  *error_msg = g_strdup_printf (_("The protocol %s is not supported."),
 					url[0]);
 	  g_strfreev (url);
-	  return NULL;
+	  return FALSE;
 	}
-      host = url[1];
+      lhost = url[1];
     }
   else
-    host = (gchar *) uri;
+    lhost = (gchar *) uri;
 
-  if (g_strrstr (host, "::") != NULL)
+  if (g_strrstr (lhost, "::") != NULL)
     {
-      server = g_strsplit (host, "::", 2);
-      port = server[1] ? atoi (server[1]) : 5900;
+      server = g_strsplit (lhost, "::", 2);
+      lport = server[1] ? atoi (server[1]) : 5900;
     }
   else
     {
-      server = g_strsplit (host, ":", 2);
-      port = server[1] ? atoi (server[1]) : 5900;
+      server = g_strsplit (lhost, ":", 2);
+      lport = server[1] ? atoi (server[1]) : 5900;
 
-      if (port < 1024)
-        port += 5900;
+      if (lport < 1024)
+        lport += 5900;
     }
 
-  host = server[0];
+  lhost = server[0];
+
+  *host = g_strdup (lhost);
+  *port = lport;
+
+  g_strfreev (server);
+  g_strfreev (url);
+
+  return TRUE;
+}
+
+VinagreConnection *
+vinagre_connection_new_from_string (const gchar *uri, gchar **error_msg)
+{
+  VinagreConnection *conn;
+  gint    port;
+  gchar  *host;
+
+  if (!vinagre_connection_split_string (uri, &host, &port, error_msg))
+    return NULL;
 
   conn = vinagre_bookmarks_exists (vinagre_bookmarks_get_default (),
                                    host,
@@ -487,9 +582,7 @@ vinagre_connection_new_from_string (const gchar *uri, gchar **error_msg)
       vinagre_connection_set_port (conn, port);
     }
 
-  g_strfreev (server);
-  g_strfreev (url);
-
+  g_free (host);
   return conn;
 }
 
@@ -589,4 +682,81 @@ vinagre_connection_get_icon (VinagreConnection *conn)
 
   return pixbuf;
 }
+
+void
+vinagre_connection_set_view_only (VinagreConnection *conn,
+				  gboolean value)
+{
+  g_return_if_fail (VINAGRE_IS_CONNECTION (conn));
+
+  conn->priv->view_only = value;
+}
+gboolean
+vinagre_connection_get_view_only (VinagreConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_CONNECTION (conn), FALSE);
+
+  return conn->priv->view_only;
+}
+
+void
+vinagre_connection_set_scaling (VinagreConnection *conn,
+				gboolean value)
+{
+  g_return_if_fail (VINAGRE_IS_CONNECTION (conn));
+
+  conn->priv->scaling = value;
+}
+gboolean
+vinagre_connection_get_scaling (VinagreConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_CONNECTION (conn), FALSE);
+
+  return conn->priv->scaling;
+}
+
+void
+vinagre_connection_set_fullscreen (VinagreConnection *conn,
+				  gboolean value)
+{
+  g_return_if_fail (VINAGRE_IS_CONNECTION (conn));
+
+  conn->priv->fullscreen = value;
+}
+gboolean
+vinagre_connection_get_fullscreen (VinagreConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_CONNECTION (conn), FALSE);
+
+  return conn->priv->fullscreen;
+}
+
+gchar*
+vinagre_connection_get_string_rep (VinagreConnection *conn,
+				   gboolean has_protocol)
+{
+  GString *uri;
+  gchar *result;
+
+  g_return_val_if_fail (VINAGRE_IS_CONNECTION (conn), NULL);
+
+  if (has_protocol)
+    {
+      uri = g_string_new (vinagre_connection_protos [conn->priv->protocol-1]);
+      g_string_append_printf (uri, "%s", "://");
+    }
+  else
+    uri = g_string_new (NULL);
+
+  g_string_append_printf (uri, "%s", conn->priv->host);
+
+  if (vinagre_connection_default_port [conn->priv->protocol-1] != conn->priv->port)
+    g_string_append_printf (uri, "::%d", conn->priv->port);
+
+  result = g_strdup (uri->str);
+  g_string_free (uri, TRUE);
+
+  return result;
+}
+
 /* vim: ts=8 */
